from translatepy.translate import *

__author__      = 'Anime no Sekai'
__copyright__   = 'Copyright 2021, translate'
__credits__     = ['animenosekai']
__license__     = 'MIT'
__version__     = 'translatepy v1.0'
__maintainer__  = 'Anime no Sekai'
__email__       = 'niichannomail@gmail.com'
__status__      = 'Beta'